#include "jeliza-console.h"
#include "../jeliza/jeliza-comm.h"

/*
 * This is part of JEliza revision 5X.
 * Copyright 2008 by Tobias Schulz
 * WWW: http://jeliza.ch.to/
 *
 * JEliza is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later
 * version.
 *
 * JEliza is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU GPL
 * along with JEliza (file COPYING) ; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin St,
 * Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <cstdlib>
#include <iostream>
#include <boost/bind.hpp>
#include <boost/smart_ptr.hpp>
#include "asio.hpp"


bool exit_after = false;
bool exit_now = false;

int main(int argc, char** argv) {
    if (argc == 1) {
        connect();
        loop();
    }
    if (argc > 1) {
        string ip = "127.0.0.1";
        string text = "";

        vector<string> args;
        for (int x = 1; x < argc; ++x) {
            args.push_back(string(argv[x]));
        }

        for (unsigned int x = 0; x < args.size(); ++x) {
            if (args[x] == "-ip") {
                ++x;

                if (x < args.size()) {
                    ip = args[x];
                }
            }
            if (args[x] == "-text") {
                ++x;

                if (x < args.size()) {
                    text = args[x];
                }
                else {
                    cerr << argv[0] << ": Invalid argument - '"  << args[--x] << "' does not have a value" << endl;
                    std::exit(1);
                }
                exit_after = true;
            }
            if (args[x] == "-exit") {
                exit_after = true;
                exit_now = true;
            }
        }

        if (text.size() == 0 && !exit_after) {
            cerr << argv[0] << ": No text given." << endl;
            std::exit(1);
        }

        jeliza::comm_init_client(ip, false, false);

        while (!ok_to_write) {
            asio::deadline_timer timer(io_service, boost::posix_time::milliseconds(100));
            timer.wait();
        }

        if (text.size()) {
            jeliza::comm_send("QUESTION:" + text);
        }

        if (exit_now) {
            jeliza::comm_send("EXIT:.");
            if (!text.size()) {
                asio::deadline_timer timer(io_service, boost::posix_time::milliseconds(1000));
                std::exit(0);
            }
        }

        while (true) {
            asio::deadline_timer timer(io_service, boost::posix_time::milliseconds(1000));
            timer.wait();
        }
    }

    return 0;
}

void connect() {
    system("clear");
    cout << endl
         << "   Artificial Intelligence Connect  " << endl
         << " ===================================" << endl
         << endl;

    cout << " - IP address (default: 127.0.0.1, this computer)?" << endl;
    cout << "> ";
    string ip_default = "127.0.0.1";
    string ip;
    getline(cin, ip);
    if (ip.size() <= 4) {
        ip = ip_default;

    }
    cout << " - IP address is '" << ip << endl;
    cout << " - Connecting to AI..." << endl;

    jeliza::comm_init_client(ip, true, false);

    cout << endl;
    cout << " - please wait..." << endl;

}



void loop() {
    string line;

    do {
        getline(cin, line);
        if (line.size() == 0) {
            cout << "Exit." << endl;
            std::exit(0);
        }

        cout << " Bitte warten...";

        jeliza::comm_send("QUESTION:" + line);
    } while (true);
}

void jeliza::comm_new(jeliza::string line) {
    if (line.substr(0, 5) == "HELLO") {
        if (!exit_after && !exit_now) {
            system("clear");
            ki_name = line.substr(6, std::string::npos).ref();
            cout << "- I am " << ki_name << endl
                 << endl;
            cout << "> " << flush;
        }
    }
    if (line.substr(0, 4) == "NAME") {
        ki_name = line.substr(6, std::string::npos).ref();
    }
    if (line.substr(0, 7) == "DISPLAY") {
        string text = line.substr(8, std::string::npos);
        if (exit_after) {
            text = text.replace_nocase(s("<br>"), s("\n"));
            text = text.replace_nocase(s("<b>"), s(""));
            text.strip();
            jeliza::strings lines;
            util::split_string(text, s("::</b>"), lines, false);
            cout << lines[lines.size() - 1].ref() << endl;
            std::exit(0);
        }
        else {
            text = text.replace_nocase(s("<br>"), s("\n"));
            text = text.replace_nocase(s("<b>"), s(""));
            text = text.replace_nocase(s("</b>"), s(""));
            text = text.replace_nocase(s("::"), s(":"));
            system("clear");
            cout << text.ref() << endl
                 << endl;
            cout << "> " << flush;
        }
    }
}


